from naman.core.models import VLan
from django.core.exceptions import ImproperlyConfigured


def assign_provisioning_vlan(machine):
  
    print("Entering assign_provisioning_vlan")
    prov_vlans = VLan.objects.filter(provisioning_purpose=True)
    if prov_vlans.count() == 0:
        raise ImproperlyConfigured("Missing provisioning vlans")

    for vlan in prov_vlans:
        try:
            machine.get_vlanconfig().append_vlan(vlan)
            return
        except VLan.NoFreeIPError:
            continue

    raise VLan.NoFreeIPError("No free IPs at any provisioning vlan")
    
  
def assign_backup_vlan(machine):
    #logging.basicConfig(level=logging.DEBUG)
    print("Entering assign_backup_vlan")
    for vlan in machine.environment.backup_vlans.all().order_by('name'):
        try:
            machine.get_vlanconfig().append_vlan(vlan)
            return
        except VLan.NoFreeIPError:
            continue
    raise VLan.NoFreeIPError("No free IPs at any backup vlan")


def assign_management_vlan(machine):
    print("Entering management vlan")
    man_vlans = VLan.objects.filter(management_purpose=True)
    if man_vlans.count() == 0:
        raise ImproperlyConfigured("Missing management vlans")

    for vlan in man_vlans:
        try:
            machine.get_vlanconfig().append_vlan(vlan)
            return
        except VLan.NoFreeIPError:
            continue

    raise VLan.NoFreeIPError("No free IPs at any management vlan")


def assign_dmz_based_on_project(machine):
    print("Entering dmz based on project vlan")
    #if machine.dmz_located:
    project = machine
    if project is None or project.dmz is None:
        raise ImproperlyConfigured(
            "DMZ located machine must belong to a project which has dmz vlan assing")
    machine.get_vlan_config().append_vlan(project.dmz)
        
        
def assign_service_vlan_based_on_project(machine):
    print("Entering service vlan based on project")
    project = machine.project
    for vlan in project.service_vlans.all().order_by('name'):
        try:
            machine.get_vlanconfig().append_vlan(vlan)
            return
        except VLan.NoFreeIPError:
            continue
    
def assign_general_purpose_service_vlan(machine):
    print("General purpose service vlan")
    for vlan in machine.environment.service_vlans.all().order_by('name'):
        #print "trying service vlan with: %s" % vlan
        try:
            machine.get_vlanconfig().append_vlan(vlan)
            return
        except VLan.NoFreeIPError:
            continue

    #raise VLan.NoFreeIPError("Can't assign free IP for service vlan")
mappings = [
    assign_backup_vlan,
    assign_management_vlan,
    assign_provisioning_vlan,
    assign_dmz_based_on_project,
    assign_service_vlan_based_on_project,
    assign_general_purpose_service_vlan,
   
    ]


from models import *

from rest_framework import serializers


class ExcludedIPRangeSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = ExcludedIPRange
        fields = ('first', 'last', 'vlan', 'id', )


class ServiceSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Service
        fields = ('name', 'description', 'iface', )


class DNSZoneSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = DNSZone
        fields = ('name', 'id', )


class VLanSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = VLan
        fields = ('name', 'ip', 'gw', 'mask', 'tag', 'id', 'management_purpose', 'provisioning_purpose', )
        lookup_field = "code"


class EnvironmentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Environment
        fields = ('code', 'description', 'id', )


class OperatingSystemSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = OperatingSystem
        fields = ('code', 'description', 'id', )


class ProjectSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Project
        fields = ('name', 'code', 'machines', 'id', )


class RoleSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Role
        fields = ('code', 'description', 'id', )


class MTypeSerializer(serializers.ModelSerializer):

    class Meta:
        model = MType


class MachineSerializer(serializers.HyperlinkedModelSerializer):
    interfaces = serializers.HyperlinkedRelatedField(many=True, read_only=True,
                                                 view_name='iface-detail')

    fqdn = serializers.Field(source="fqdn")
    class Meta:
        model = Machine
        fields = (
            "hostname",
            "virtual",
            'environment',
            'role',
            'operating_system',
            "dns_zone",
            "project",
            "mtype",
            "interfaces",
            "id",
            "fqdn",)


class IfaceSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Iface
        fields = (
            'name',
            'ip',
            'vlan',
            'gw',
            'mask',
            'machines',
            'id',
            'virtual',
            'nat',
            'mac',
            "dhcp",)


class VLanConfigSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = VLanConfig




from django.shortcuts import render_to_response
from django.contrib.auth.decorators import user_passes_test
from django.template import RequestContext
# Create your views here.


@user_passes_test(lambda u: u.is_staff)
def home(request):
    return render_to_response(
        "base.html",
        {},
        context_instance=RequestContext(request))



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
#from django.test import LiveServerTestCase
#from django.test import TestCase
from django.utils import unittest
from selenium.webdriver.firefox.webdriver import WebDriver
from models import *
import requests
from requests.auth import HTTPBasicAuth
import os
from django.conf import settings
from django.contrib.auth.models import User
import simplejson
from django.test.client import RequestFactory
from views import (
    ProjectViewSet,
    MachineViewSet,
    VLanViewSet,
    IfaceViewSet,
    ExcludedIPRangeViewSet)
from rest_framework.reverse import reverse
from django.test import TestCase
from django.db import IntegrityError
from django.core.exceptions import ImproperlyConfigured
import ipaddr


def add_iface_for_machine(machine, vlan):
    return machine.add_iface(vlan)


def add_project(name, code):

    p = Project(name=name, code=code)
    p.save()
    return p


def add_machine(project, dns_zone, env, role, os, mtype, virtual=True, dmz_located=False):
    m = Machine(
        hostname="",
        dns_zone=dns_zone,
        environment=env,
        role=role,
        operating_system=os,
        virtual=virtual,
        project=project,
        dmz_located=dmz_located,
        mtype=mtype,
    )
    m.save()

    return m


def create_iface_creation_request(user, machine, vlan, ip=None):
    requestfactory = RequestFactory()
    data = simplejson.dumps({
        "vlan": reverse("vlan-detail", args=[vlan.pk, ]),
        "ip": ip,
        "machines": [reverse("machine-detail", args=[machine["id"], ], ), ],

    })
    request = requestfactory.post(
        reverse("iface-list"),
        data=data,
        content_type="application/json", )

    request.user = user
    request.session = {}
    print data
    return request


def create_machine_creation_request(user, project, dns_zone, env, role, os, hostname=None, mtype=None):
    requestfactory = RequestFactory()

    project = reverse(
        "project-detail",
        args=[project.pk, ]) if project is not None else ""

    data = simplejson.dumps({
        "project": project,
        "role": reverse("role-detail", args=[role.pk, ]),
        "operating_system": reverse(
            "operatingsystem-detail",
            args=[os.pk, ]),
        "environment": reverse(
            "environment-detail",
            args=[env.pk, ]),
        "dns_zone": reverse("dnszone-detail", args=[dns_zone.pk, ]),
        "mtype": reverse("mtype-detail", args=[mtype.pk, ]),
        "hostname": hostname,
    })

    request = requestfactory.post(
        reverse("machine-list"),
        data=data,
        content_type="application/json", )

    request.user = user
    request.session = {}

    return request


def add_vlan(name, tag, ip, gw, mask=24):
    v = VLan(
        name=name,
        tag=tag,
        ip=ip,
        gw=gw,
        mask=mask,
    )
    v.save()
    return v


def add_os(code, descr):
    os = OperatingSystem(
        code=code,
        description=descr,
    )
    os.save()
    return os


def add_role(code, descr):
    r = Role(
        code=code,
        description=descr,
    )
    r.save()
    return r


def add_env(code, descr):
    e = Environment(
        code=code,
        description=descr
    )
    e.save()
    return e


def add_dnszone(name):
    dz = DNSZone(name=name)
    dz.save()
    return dz


def add_mtype(name, auto_name, has_serial):
    mt = MType(name=name, auto_name=auto_name, has_serial=has_serial)
    mt.save()

    return mt


def add_superuser(name, password):
    u = User(username=name, password=password, is_superuser=True)
    u.save()
    return u


def create_excluded_ip_range_request(user, first, last, vlan):
    requestfactory = RequestFactory()
    data = simplejson.dumps({
        "vlan": reverse("vlan-detail", args=[vlan.pk, ]),
        "first": first,
        "last": last,
    })
    request = requestfactory.post(
        reverse("excludediprange-list"),
        data=data,
        content_type="application/json", )

    request.user = user
    request.session = {}
    print data
    return request


class MachineTest(TestCase):

    def setUp(self, ):
        self.factory = RequestFactory()
        self.z_ib = add_dnszone(".ib")
        self.env_pro = add_env("PRO", "production")
        self.env_pre = add_env("PRE", "preproduction")
        self.env_des = add_env("DES", "development")
        self.role_bd = add_role("BD", "base de datos")
        self.role_bd.needs_backup_vlan = True
        self.role_bd.save()

        self.role_sa = add_role("SA", "Servidor aplicaciones")
        self.role_sw = add_role("SW", "Servidor Web")
        self.role_ut = add_role("UT", "Utils server")
        self.role_ce = add_role("CE", "Conmutador Electronico")
        self.role_bal = add_role("BL", "Load Balancer")
        self.os_lin = add_os("L", "GNU/Linux")
        self.os_win = add_os("W", "Windows")

        self.mtype_server = add_mtype(
            "server",
            auto_name=True,
            has_serial=True)
        self.mtype_router = add_mtype(
            "router",
            auto_name=False,
            has_serial=False)
        self.mtype_server_standalone = add_mtype(
            "server_standalone",
            auto_name=True,
            has_serial=False)

        self.vlan_man1 = add_vlan(
            "MAN1",
            200,
            "172.21.200.0",
            "172.21.200.1",
            23,
        )
        self.vlan_patio = add_vlan(
            "PATIO",
            229,
            "172.21.228.0",
            "172.21.228.1",
            23,
        )
        self.project_ibcom = add_project("IBC", "IBC")
        self.project_crm = add_project("CRM", "CRM")
        try:

            self.user = User.objects.get(username="raton")
        except:
            self.user = add_superuser("raton", "r")

    def test_find_ip_exclusion(self, ):
        ExcludedIPRange.objects.all().delete()
        eir = ExcludedIPRange(
            first="172.21.229.1",
            last="172.21.229.10",
            vlan=self.vlan_patio,
        )
        eir.save()

        self.assertEquals(
            Iface.excluded_in_ranges("172.21.229.5"),
            [eir, ],
            "Not properly finding Excluded IP Ranges for IP 172.21.229.5, should: %s" % eir,
        )

        eir1 = ExcludedIPRange(
            first="172.21.229.8",
            last="172.21.229.12",
            vlan=self.vlan_patio,
        )
        eir1.save()

        self.assertEquals(
            Iface.excluded_in_ranges("172.21.229.9"),
            [eir, eir1, ],
            "Not properly finding Excluded IP Ranges for IP 172.21.229.9, should: %s" % [eir, eir1, ],
        )

        self.assertEquals(
            Iface.excluded_in_ranges("172.21.210.9"),
            [],
            "Not properly finding Excluded IP Ranges for IP 172.21.210.9, should None" % [eir, eir1, ],
        )

    def test_excluded_ip_ranges(self, ):

        #create an ip range who's first IP is not suitable for the assigned vlan
        request = create_excluded_ip_range_request(
            self.user,
            "172.21.200.10",
            "172.21.229.10",
            self.vlan_patio)

        func = ExcludedIPRangeViewSet.as_view({"post": "create"})
        self.assertRaises(
            ipaddr.AddressValueError,
            func, request,
            "creating an excluded ip range, not properly checking if first ip is valid for assigned vlan")

        #create an ip range who's second IP is not suitable for the assigned vlan
        request = create_excluded_ip_range_request(
            self.user,
            "172.21.229.10",
            "172.21.200.10",
            self.vlan_patio)

        self.assertRaises(
            ipaddr.AddressValueError,
            func, request,
            "creating an excluded ip range, not properly checking if second ip is valid for assigned vlan")

        #create an ip range correctly for vlan addressing with one IP
        request = create_excluded_ip_range_request(
            self.user,
            "172.21.228.10",
            "172.21.228.10",
            self.vlan_patio)

        response = (ExcludedIPRangeViewSet.as_view({"post": "create"})(request)).render()
        eir = simplejson.loads(response.content)

        self.assertEquals(
            201,
            response.status_code,
            "Is not properly creating excluded_ip_ranges (single IP)")

        #create an ip range correctly for vlan addressing with more than one IP
        request = create_excluded_ip_range_request(
            self.user,
            "172.21.228.1",
            "172.21.228.8",
            self.vlan_patio)

        response = (ExcludedIPRangeViewSet.as_view({"post": "create"})(request)).render()
        eir = simplejson.loads(response.content)
        self.assertEquals(
            201,
            response.status_code,
            "Is not properly creating excluded_ip_ranges (more than one IP)")

        #does it respect excluded ranges at iface creation??
        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server)
        machine = simplejson.loads(
            (MachineViewSet.as_view({"post": "create"})(request)).render().content)

        request = create_iface_creation_request(
            self.user,
            machine,
            self.vlan_patio)
        print request
        response = IfaceViewSet.as_view({"post": "create"})(request)
        iface = simplejson.loads(response.render().content)

        print iface
        self.assertEquals(
            iface["ip"],
            "172.21.228.9",
            "Creating an iface is not looking at created excluded_ip_ranges")

        request = create_iface_creation_request(
            self.user,
            machine,
            self.vlan_patio)

        response = IfaceViewSet.as_view({"post": "create"})(request)
        iface = simplejson.loads(response.render().content)

        print iface
        self.assertEquals(
            iface["ip"],
            "172.21.228.11",
            "Creating an iface is not looking at created excluded_ip_ranges")

    def off_test_many_machines_for_one_iface(self, ):
        #TODO write test_many_machines_for_one_iface
        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server)
        machine1 = simplejson.loads(
            (MachineViewSet.as_view({"post": "create"})(request)).render().content)

        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server)
        machine2 = simplejson.loads(
            (MachineViewSet.as_view({"post": "create"})(request)).render().content)

        request = create_iface_creation_request(
            self.user,
            machine1,
            self.vlan_man1)
        response = IfaceViewSet.as_view({"post": "create"})(request)

        iface = simplejson.loads(response.render().content)

        requestfactory = RequestFactory()
        data = simplejson.dumps({
            "machine": reverse("iface-detail", args=[machine1["id"], ]),
        })
        request = requestfactory.post(
            reverse("iface-list"),
            data=data,
            content_type="application/json", )

        request.user = user
        request.session = {}
        print data

    def test_interface_rest(self, ):
        #regular iface creation with IP
        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server)
        machine = simplejson.loads(
            (MachineViewSet.as_view({"post": "create"})(request)).render().content)

        request = create_iface_creation_request(
            self.user,
            machine,
            self.vlan_man1,
            "6.6.6.6",)

        response = IfaceViewSet.as_view({"post": "create"})(request)
        response.render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly creating iface. (should: 201; does: %s) %s" %
                  (response.status_code, response.content))

        self.assertEquals(
            simplejson.loads(response.content)["ip"],
            "6.6.6.6",
            "Not properly creating ifaces when IP is assigned by user")

        # creating the same vlan for the same machine, should create it
        request = create_iface_creation_request(
            self.user,
            machine,
            self.vlan_man1)
        response = IfaceViewSet.as_view({"post": "create"})(request)
        self.assertEquals(
            201,
            response.status_code,
            "Not properly creating second iface on same machine and vlan. (should: 201; does: %s): %s" %
                (response.status_code, response.render().content))

        #regular iface creation without IP
        request = create_iface_creation_request(
            self.user,
            machine,
            self.vlan_man1)

        response = IfaceViewSet.as_view({"post": "create"})(request)
        response.render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly creating iface. (should: 201; does: %s) %s" %
                (response.status_code, response.content))

    def test_conflicting_ip(self, ):
        Iface.objects.all().delete()
        ExcludedIPRange.objects.filter(vlan=self.vlan_man1).delete()
        ConflictingIP(ip="172.21.200.1").save()
        ip = self.vlan_man1.get_ip()
        self.assertEquals(
            ip,
            "172.21.200.2",
            "Not properly assigning an IP when there is a conflicting one. (should: 172.21.200.2, does: %s)" % ip
        )

    def test_find_vlan_for_ip(self, ):
        VLan.objects.all().delete()
        vlan_servicio = add_vlan(
            "SERVICIO",
            100,
            "172.21.100.0",
            "172.21.100.1",
            24,
        )

        vlan = Iface.find_vlan("192.168.1.1")
        self.assertIsNone(
            vlan,
            "Should return None because no 192.168.1.1 does not belong to any vlan, returned: %s" % vlan
        )

        self.assertEqual(
            vlan_servicio,
            Iface.find_vlan("172.21.100.3"),
            "Should return %s as 172.21.100.3 belongs to it" % vlan_servicio
        )

    def test_vlan_decision(self, ):
        #cleaning ifaces *******************************
        Iface.objects.all().delete()

        vlan_servicio = add_vlan(
            "SERVICIO",
            100,
            "172.21.100.0",
            "172.21.100.1",
            30,
        )

        vlan_dmz = add_vlan(
            "DMZ",
            101,
            "172.21.101.0",
            "172.21.101.1",
            30,
        )

        vlan_management = add_vlan(
            "MNG",
            102,
            "172.21.102.0",
            "172.21.102.1",
            30,
        )
        vlan_management.management_purpose = True
        vlan_management.save()

        print vlan_management.info

        vlan_project = add_vlan(
            "PROJECT",
            103,
            "172.21.103.0",
            "172.21.103.1",
            30,
        )
        vlan_backup_prod = add_vlan(
            "backup_prod",
            104,
            "172.21.104.0",
            "172.21.104.1",
            30,
        )
        vlan_backup_des = add_vlan(
            "backup_des",
            105,
            "172.21.105.0",
            "172.21.105.1",
            30,
        )

        vlan_pre = add_vlan(
            "pre",
            106,
            "172.21.106.0",
            "172.21.106.1",
            30,)

        self.env_des.backup_vlans.add(vlan_backup_des)
        self.env_pre.service_vlans.add(vlan_pre)
        self.env_pre.backup_vlans.add(vlan_backup_des)
        self.env_pro.backup_vlans.add(vlan_backup_prod)
        self.env_pro.service_vlans.add(vlan_servicio)

        project = add_project('Federation Project', 'fed')
        project.service_vlans.add(vlan_project)
        project.dmz = vlan_dmz
        project.save()

        #creating vlan config
        #  project machine
        #  campus network
        #  production
        #  role BD

        # should assign vlans: vlan_project, vlan_backup_prod, vlan_management
        m1 = add_machine(
            project,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,
        )

        vc.save()
        print m1
        print vc.vlans.all().order_by('name')
        self.assertEquals(
            [vlan_management, vlan_project, vlan_backup_prod, ],
            [x for x in vc.vlans.all().order_by('name')],
            "Should have vlan_backup_prod, vlan_management, vlan_project")

        for v in vc.vlans.all():
            i = Iface(vlan=v)
            i.save()
            i.machines.add(m1)

        #creating second vlan config
        #  project machine
        #  campus network
        #  production
        #  role BD

        # should assign vlans: vlan_project, vlan_backup_prod, vlan_management
        m1 = add_machine(
            project,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,
        )

        vc.save()

        self.assertEquals(
            [vlan_management, vlan_project, vlan_backup_prod, ],
            [x for x in vc.vlans.all().order_by('name')],
            "Should have vlan_backup_prod, vlan_management, vlan_project")

        for v in vc.vlans.all():
            i = Iface(vlan=v)
            i.save()
            i.machines.add(m1)

        #creating third vlan config
        #  project machine
        #  campus network
        #  production
        #  role BD

        # should not find free IPs
        m1 = add_machine(
            project,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,
        )
        func = vc.save
        self.assertRaises(
            VLan.NoFreeIPError,
            func,
            "Should raise NoFeeIPError as there are no free IPs"
        )

        #cleaning ifaces *******************************
        Iface.objects.all().delete()

        #creating vlan config
        #  NON project machine
        #  campus network
        #  production
        #  role SA(no backup needed)

        # should assign vlans: vlan_servicio, vlan_management
        m1 = add_machine(
            None,
            self.z_ib,
            self.env_pro,
            self.role_sa,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,
        )

        vc.save()
        self.assertEquals(
            [vlan_management, vlan_servicio, ],
            [x for x in vc.vlans.all().order_by('name')],
            "Should have vlan_servicio and vlan_management")

        Iface(vlan=vlan_servicio).save()
        Iface(vlan=vlan_servicio).save()
        #creating vlan config
        #  NON project machine
        #  campus network
        #  production
        #  role SA(no backup needed)

        # should not find free IPs for vlan_servicio
        m1 = add_machine(
            None,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,)

        func = vc.save
        self.assertRaises(
            VLan.NoFreeIPError,
            func,
            "It should find free IPs for vlan_servicio, but it did")
        #cleaning ifaces *******************************
        Iface.objects.all().delete()

        Iface(vlan=vlan_backup_prod).save()
        Iface(vlan=vlan_backup_prod).save()
        #creating vlan config
        #  NON project machine
        #  campus network
        #  production
        #  role SA(no backup needed)

        # should not find free IPs for vlan_backup
        m1 = add_machine(
            None,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,)

        func = vc.save
        self.assertRaises(
            VLan.NoFreeIPError,
            func,
            "It should find free IPs for vlan_backup_prod, but it did")

        #cleaning ifaces *******************************
        Iface.objects.all().delete()

        #creating vlan config
        #  NON project machine
        #  campus network
        #  pre
        #  role SA(no backup needed)

        # should not find free IPs for vlan_backup
        m1 = add_machine(
            None,
            self.z_ib,
            self.env_pre,
            self.role_sa,
            self.os_lin,
            self.mtype_server,
            True,
            False)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,)

        vc.save()
        self.assertEquals(
            2,
            vc.vlans.all().count(),
            "Should onkly have 2 vlans, as role needs no backup")

        self.assertEquals(
            [vlan_management, vlan_pre, ],
            [x for x in vc.vlans.all().order_by('name')],
            "Should have vlan_pre and vlan_management")

        #creating vlan config
        #  project machine
        #  DMZ network
        #  pro
        #  role SA(no backup needed)

        # should assign [vlan_management, vlan_dmz]
        m1 = add_machine(
            project,
            self.z_ib,
            self.env_pro,
            self.role_sa,
            self.os_lin,
            self.mtype_server,
            True,
            True)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,)

        vc.save()
        self.assertEquals(
            2,
            vc.vlans.all().count(),
            "Should onkly have 2 vlans, as role needs no backup")

        self.assertEquals(
            [vlan_dmz, vlan_management, ],
            [x for x in vc.vlans.all().order_by('name')],
            "Should have vlan_dmz and vlan_management; does: %s" % vc)

        #creating vlan config
        #  NON project machine
        #  DMZ network
        #  pro
        #  role SA(no backup needed)

        # should raise ImproperConfigured Exception because system
        # can't find dmz for non-project machine
        m1 = add_machine(
            None,
            self.z_ib,
            self.env_pro,
            self.role_sa,
            self.os_lin,
            self.mtype_server,
            True,
            True)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,)

        func = vc.save
        self.assertRaises(
            ImproperlyConfigured,
            func,
            "Should raise ImproperlyConfigured because system can't allocate dmz for non-project machine")

        #creating vlan config
        #  project machine
        #  DMZ network
        #  pro
        #  role BD

        # should [vlan_dmz, vlan_backup, vlan_management]
        m1 = add_machine(
            project,
            self.z_ib,
            self.env_pro,
            self.role_bd,
            self.os_lin,
            self.mtype_server,
            True,
            True)

        vc = VLanConfig(
            machine=m1,
            needs_backup=True,
            needs_management=True,)

        vc.save()
        self.assertEquals(
            3,
            vc.vlans.all().count(),
            "Should have 3 vlans, as role needs backup")

        self.assertEquals(
            [vlan_dmz, vlan_management, vlan_backup_prod],
            [x for x in vc.vlans.all().order_by('name')],
            "Should have vlan_dmz and vlan_management; does: %s" % vc)

    def test_standalone_hostnames_rest(self, ):

        #creating a regular server for CRM project
        request = create_machine_creation_request(
            self.user,
            self.project_crm,
            self.z_ib,
            self.env_des,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server,
        )
        response = MachineViewSet.as_view({"post": "create"})(request).render()
        proper_name = ("%s%s%s%s1" % (
            self.role_bd.code,
            self.project_crm.code,
            self.os_lin.code,
            self.env_des.code)).lower()

        machine = simplejson.loads(response.content)
        self.assertEquals(
            proper_name,
            machine["hostname"],
            "Not properly creating cluster server hostname (should: %s; does: %s)" % (
                proper_name,
                machine["hostname"],
            )
        )
        #creating an standalone server once already exists a regular server with same properties
        request = create_machine_creation_request(
            self.user,
            self.project_crm,
            self.z_ib,
            self.env_des,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server_standalone
        )
        response = MachineViewSet.as_view({"post": "create"})(request).render()
        proper_name = ("%s%s%s%s" % (
            self.role_bd.code,
            self.project_crm.code,
            self.os_lin.code,
            self.env_des.code)).lower()

        machine = simplejson.loads(response.content)
        self.assertEquals(
            proper_name,
            machine["hostname"],
            "Not properly creating standalone server hostname when a cluster server exist for the same properties (should: %s; does: %s)" % (
                proper_name,
                machine["hostname"],
            )
        )

        #creating the second regular server once already exists a
        #regular server and a standalone server with same properties
        request = create_machine_creation_request(
            self.user,
            self.project_crm,
            self.z_ib,
            self.env_des,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server
        )
        response = MachineViewSet.as_view({"post": "create"})(request).render()
        proper_name = ("%s%s%s%s2" % (
            self.role_bd.code,
            self.project_crm.code,
            self.os_lin.code,
            self.env_des.code)).lower()

        machine = simplejson.loads(response.content)
        self.assertEquals(
            proper_name,
            machine["hostname"],
            "Not properly creating hostnames (should: %s; does: %s)" % (
                proper_name,
                machine["hostname"],
            )
        )

    def off_test_no_auto_name(self, ):

        #creating machine without autoname and not sending hostname
        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_ce,
            self.os_lin,
            mtype=self.mtype_router)

        try:
            MachineViewSet.as_view({"post": "create"})(request)
            assert("No properly creating machine with no auto name. No hostname has been sent, it should return 400, does: %s" % response.status_code)
        except IntegrityError:
            pass

        #creating machine that should increment counter
        request = create_machine_creation_request(
            self.user,
            self.project_ibcom,
            self.z_ib,
            self.env_des,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server)

        response = MachineViewSet.as_view({"post": "create"})(request).render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly return status after machine creation(should:201;does:%s" % response.status_code)

        proper_name = ("%s%s%s%s1" % (
            self.role_bd.code,
            self.project_ibcom.code,
            self.os_lin.code,
            self.env_des.code)).lower()

        machine = simplejson.loads(response.content)
        self.assertEquals(
            proper_name,
            machine["hostname"],
            "Not properly creating hostnames (should: %s; does: %s)" % (
                proper_name,
                machine["hostname"],
            )
        )

        # creating machine that needs a specified hostname, even though there is
        # a previous cluster server with same properties
        request = create_machine_creation_request(
            self.user,
            self.project_ibcom,
            self.z_ib,
            self.env_des,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server_standalone,
            hostname="bdibcldes")

        response = MachineViewSet.as_view({"post": "create"})(request).render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly return status after machine creation(should:201;does:%s)" % response.status_code)

        proper_name = "bdibcldes"

        machine = simplejson.loads(response.content)
        self.assertEquals(
            proper_name,
            machine["hostname"],
            "Not properly creating hostnames (should: %s; does: %s)" % (
                proper_name,
                machine["hostname"],
            )
        )
        #creating the other cluster machine
        request = create_machine_creation_request(
            self.user,
            self.project_ibcom,
            self.z_ib,
            self.env_des,
            self.role_bd,
            self.os_lin,
            mtype=self.mtype_server)

        response = MachineViewSet.as_view({"post": "create"})(request).render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly return status after machine creation(should:201;does:%s" % response.status_code)

        proper_name = ("%s%s%s%s2" % (
            self.role_bd.code,
            self.project_ibcom.code,
            self.os_lin.code,
            self.env_des.code)).lower()

        machine = simplejson.loads(response.content)
        self.assertEquals(
            proper_name,
            machine["hostname"],
            "Not properly creating hostnames (should: %s; does: %s)" % (
                proper_name,
                machine["hostname"],
            )
        )

    def test_alta_router(self, ):
        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_ce,
            self.os_lin,
            mtype=self.mtype_router,
            hostname="CE_MAN1"
        )
        response = MachineViewSet.as_view({"post": "create"})(request).render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly return status after machine creation(should:201;does:%s" % response.status_code)

        machine = simplejson.loads(response.content)
        self.assertEquals(
            "CE_MAN1",
            machine["hostname"],
            "Not properly creating hostnames (should: %s; does: %s)" % (
                "CE_MAN1",
                machine["hostname"],
            )
        )



    def test_alta_balanceador(self, ):
        request = create_machine_creation_request(
            self.user,
            None,
            self.z_ib,
            self.env_pro,
            self.role_bal,
            self.os_lin,
            mtype=self.mtype_router,
            hostname="bigip1"
        )
        response = MachineViewSet.as_view({"post": "create"})(request).render()
        self.assertEquals(
            201,
            response.status_code,
            "Not properly return status after load balancer creation(should:201;does:%s" % response.status_code)
        machine = simplejson.loads(response.content)
        self.assertEquals(
            machine["hostname"],
            "bigip1",
            "Not properly initializing load balancer hostname",
        )



from django.template import Library


register = Library()


@register.inclusion_tag("forms/filters.html")
def filters(filtro, action="", name=None, ajax_container=None):
    if name == None:
        name = "fil"

    return {"action": action, "filtro": filtro, "name": name, "ajax_container": ajax_container}





def environment(machine):
    
    env = machine.environment.to_pypelib()
    print("Environment: %s" % env)
    return env


def project(machine):
    return machine.project.to_pypelib()


def role_needs_backup(machine):
    does = "1" if machine.role.needs_backup_vlan else "0"
    print("role_needs_backup: %s" % does)
    return does
    
    
def needs_backup(machine):
    does = "1" if machine.get_vlanconfig().needs_backup else "0"
    print("vlan_config needs backup: %s" % does)
    return does

    
def dmz_located(machine):
    does = "1" if machine.dmz_located else "0"
    print("machine dmz located: %s" % does)
    return does


def needs_management(machine):
    does = "1" if machine.get_vlanconfig().needs_management else "0"
    print("Vlan_config needs management: %s" % does)
    return does

    
mappings = [
    environment,
    project,
    role_needs_backup,
    dmz_located,
    needs_management,
    needs_backup,
   ]


from django.contrib import admin
from models import *


class ConditionalAdmin(admin.ModelAdmin):
    pass
admin.site.register(Conditional, ConditionalAdmin)


class RuleAdmin(admin.ModelAdmin):
    pass
admin.site.register(Rule, RuleAdmin)


class ExcludedIPRangeAdmin(admin.ModelAdmin):
    pass
admin.site.register(ExcludedIPRange, ExcludedIPRangeAdmin)


class ConflictingIPAdmin(admin.ModelAdmin):
    pass
admin.site.register(ConflictingIP, ConflictingIPAdmin)


class VLanConfigAdmin(admin.ModelAdmin):
    pass
admin.site.register(VLanConfig, VLanConfigAdmin)


class ServiceAdmin(admin.ModelAdmin):
    pass
admin.site.register(Service, ServiceAdmin)


class ProjectAdmin(admin.ModelAdmin):
    pass
admin.site.register(Project, ProjectAdmin)


class MTypeAdmin(admin.ModelAdmin):
    pass
admin.site.register(MType, MTypeAdmin)


class MachineAdmin(admin.ModelAdmin):
    pass
admin.site.register(Machine, MachineAdmin)


class DNSZoneAdmin(admin.ModelAdmin):
    pass
admin.site.register(DNSZone, DNSZoneAdmin)


class IfaceAdmin(admin.ModelAdmin):
    pass
admin.site.register(Iface, IfaceAdmin)


class EnvironmentAdmin(admin.ModelAdmin):
    pass
admin.site.register(Environment, EnvironmentAdmin)


class RoleAdmin(admin.ModelAdmin):
    pass
admin.site.register(Role, RoleAdmin)


class OperatingSystemAdmin(admin.ModelAdmin):
    pass
admin.site.register(OperatingSystem, OperatingSystemAdmin)


class VLanAdmin(admin.ModelAdmin):
    pass
admin.site.register(VLan, VLanAdmin)



#from django.contrib.auth.decorators import login_required
from django.shortcuts import render


#@login_required
def iface(request):
    return render(request, 'js/iface.js', content_type='application/javascript')


def vlanconfig(request):
    return render(request, 'js/vlanconfig.js', content_type='application/javascript')


def conflicting_ip(request):
    return render(request, 'js/conflicting_ip.js', content_type='application/javascript')


def machine(request):
    return render(request, 'js/machine.js', content_type='application/javascript')



from django.template import Library
from urlparse import urlparse, parse_qsl
from urllib import urlencode


register = Library()


@register.inclusion_tag("forms/filters.html")
def filters(filtro, action="", name=None, ajax_container=None):
    if name == None:
        name = "fil"

    return {"action": action, "filtro": filtro, "name": name, "ajax_container": ajax_container}


@register.inclusion_tag("forms/label.html")
def label(field, contents=None, attrs=None):
    if isinstance(attrs, basestring):
        attrs = [tuple(pair.split('=')) for pair in attrs.split(',')]
    attrs = dict(attrs or {})
    widget = field.field.widget
    for_id = widget.id_for_label(widget.attrs.get('id') or field.auto_id)
    contents = contents or field.label
    return {'field': field, 'id': for_id, 'contents': contents, 'attrs': attrs}


@register.inclusion_tag("forms/bootstrap_field.html")
def bootstrap_field(field, form_group_class=None, label_class=None, inner_div_class=None, input_field_class=None):
    widget = field.field.widget
    for_id = widget.id_for_label(widget.attrs.get('id') or field.auto_id)
    if input_field_class is not None:
        field.field.widget.attrs["class"] = input_field_class
    return {
        'field': field,
        'for_id': for_id,
        "form_group_class": form_group_class,
        "label_class": label_class,
        "inner_div_class": inner_div_class,
        }


@register.filter
def or_none(value):
    if value == None:
        return "-"
    return value


@register.filter
def attrlist(attrs):
    attrs = dict(attrs)

    ret = ""
    for k, v in attrs.items():
        ret += "%s=%s" % (k, v)
    return ret
attrlist.is_safe = True


@register.inclusion_tag("forms/pagination.html")
def paginator(paginator, objeto=None, urll=None):
    print "LA url es: %s" % urll
    return {
        "paginator": paginator,
        "objeto": objeto if objeto not in (None, "") else "P&aacute;gina",
        "urll": urll if urll != None else "",
        }


@register.inclusion_tag("forms/url_add_query_element.html")
def url_add_query_element(url, key, value):
    """
    appends a query element to an existing url
    """
    u = urlparse(url)
    data = dict(parse_qsl(u.query))

    data[key] = value

    return {"url": "%s?%s" % (u.path, urlencode(data))}


@register.inclusion_tag("forms/pagination_load.html")
def paginator_load(paginator, container, objeto=None, url=None):

    return {
        "paginator": paginator,
        "objeto": objeto if objeto != None else "P&aacute;gina",
        "url": url if url != None else "",
        "container": container,
        }


@register.filter('field_type')
def field_type(ob):
    return ob.field.widget.__class__.__name__



from naman.core.models import VLanConfig
from forms import VLanConfigForm#, IfaceByMachineForm
from django.shortcuts import render_to_response, redirect#, get_object_or_404
from django.template import RequestContext
#from tools.views import paginator
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.http import HttpResponseServerError, HttpResponseBadRequest
from django.contrib.auth.decorators import user_passes_test
import simplejson


@user_passes_test(lambda u: u.is_staff)
def edit(request, id):
    try:
        obj = VLanConfig.objects.get(id=id)
    except VLanConfig.DoesNotExist:
        obj = VLanConfig()

    try:
        wrong = False
        if request.method == "POST":
            form = VLanConfigForm(request.POST, instance=obj)
            try:
                if form.is_valid():
                    obj = form.save(commit=False)
                    obj.save()
                    #ifaces = []
                    #for vlan in obj.vlans.all():
                    #    iface = Iface(vlan=vlan)
                    #    iface.save()
                    #    iface.machines.add(obj.machine)
                    return redirect(reverse("ifaces_by_machine", args=[obj.machine.pk, ]))
    
                response = render_to_response(
                    'vlanconfig/edit.html',
                    {
                        'form': form,
                        "id": obj.pk,
                        "obj": obj,
                     },
                    context_instance=RequestContext(request))
                if wrong:
                    response.status_code = 400
                return response
            except Exception, ex:                
                return HttpResponseServerError(ex)
            wrong = True
            messages.error(request, "Wrong fields!")
    
        else:
            print request.GET
            if "machine" in request.GET:
                machine = request.GET["machine"]
                form = VLanConfigForm(instance=obj, initial={"machine": machine})
            else:
                form = VLanConfigForm(instance=obj)
    
        return render_to_response(
                'vlanconfig/edit.html',
                {
                    'form': form,
                    "id": obj.pk,
                    "obj": obj,
                 },
                context_instance=RequestContext(request))
    except Exception, ex:
        
        return HttpResponseBadRequest(
            content=simplejson.dumps({"errors": ["Error: %s" % ex, ]}),
            content_type="application/json",
            )



from django.conf.urls import url, patterns, include


urlpatterns = patterns("",
    url(r"^machine/delete/(\d+)?/?", "naman.web.views_machine.delete", name="delete_machine"),
    url(r"^machine/(\d+)?/?", "naman.web.views_machine.edit", name="machine"),
    url(r"^machines", "naman.web.views_machine.listado", name="machines"),

     url(r"^service/(\d+)?/?", "naman.web.views_service.edit", name="service"),

    #url(r"^service/(\d+)?/?", "naman.web.views_machine.edit", name="machine"),


    url(r"^iface/delete/(\d+)?/?", "naman.web.views_iface.delete", name="delete_iface"),
    #url(r"^iface_short/(\d+)?/?", "web.views_iface.edit_short", name="iface_short"),
    url(r"^iface_by_machine/(\d+)?/?", "naman.web.views_iface.edit_by_machine", name="iface_by_machine"),
    url(r"^iface/(\d+)?/?", "naman.web.views_iface.edit", name="iface"),
    url(r"^ifaces_by_machine_json/(\d+)?/?(\d+)?", "naman.web.views_iface.list_by_machine_json", name="ifaces_by_machine_json"),
    url(r"^ifaces_by_machine/(\d+)?/?(\d+)?", "naman.web.views_iface.list_by_machine", name="ifaces_by_machine"),
    url(r"^ifaces", "naman.web.views_iface.listado", name="ifaces"),

    url(r"^conflictingip/(\d+)?/?", "naman.web.views_conflicting_ip.edit_modal", name="conflictingip_modal"),

    url(r"^vlanconfig/(\d+)?/?", "naman.web.views_vlanconfig.edit", name="vlanconfig"),

    url('^conflicting_ip.js$', 'naman.web.views_js.conflicting_ip', name='conflicting_ip_js'),
    url('^iface.js$', 'naman.web.views_js.iface', name='iface_js'),
    url('^machine.js$', 'naman.web.views_js.machine', name='machine_js'),
    url('^vlanconfig.js$', 'naman.web.views_js.vlanconfig', name='vlanconfig_js'),

    url('^login/?$', 'django.contrib.auth.views.login', name='login'),
    url('^logout/', 'django.contrib.auth.views.logout', {'next_page':'/login?next=/'}, name='logout'),
    url(r"^", "naman.web.views.home", name="home"),
)




from naman.core.models import Iface, ConflictingIP
from forms import IfaceForm, IfaceShortForm, IfaceByMachineForm
from django.shortcuts import render_to_response, get_object_or_404, redirect, render
from django.template import RequestContext
from naman.core.tools.views import paginator
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponse, HttpResponseServerError
import ipaddr
from django.views.decorators.csrf import csrf_exempt
from django.contrib import messages
from django.db import IntegrityError
from django.utils.datastructures import MultiValueDictKeyError
from django.contrib.auth.decorators import user_passes_test


@user_passes_test(lambda u: u.is_staff)
def delete(request, id):
    get_object_or_404(Iface, pk=id).delete()
    return HttpResponse("OK")


@user_passes_test(lambda u: u.is_staff)
def list_by_machine_json(request, machine_id, iface_id=None):

    if iface_id is None:
        ifaces = Iface.objects.filter(machines__id=machine_id)
    else:
        ifaces = Iface.objects.filter(id=iface_id)

    ifaces = [iface.to_json() for iface in ifaces]
    return render(
        request,
        'iface/list.json',
        {"ifaces": ifaces},
        content_type='application/json')


@user_passes_test(lambda u: u.is_staff)
def list_by_machine(request, machine_id, iface_id=None):

    if iface_id is None:
        ifaces = Iface.objects.filter(machines__id=machine_id)
    else:
        ifaces = Iface.objects.filter(id=iface_id)



    return render_to_response(
        "iface/list_by_machine.html",
        {"ifaces": ifaces,
         "iface_id": iface_id},
        context_instance=RequestContext(request))


@user_passes_test(lambda u: u.is_staff)
def listado(request):
    if "query_iface" in request.GET and request.GET["query_iface"] != "":
        query_sent = request.GET["query_iface"]

        query = Iface.objects.query_cmd(query_sent)
        if query is None:
            query = Iface.objects.filter(
                Q(vlan__name__iregex=query_sent)
                | Q(ip__iregex=query_sent)
                |Q(mac__icontains=query_sent))
    else:
        query = Iface.objects.all()
        query_sent = None
    print query.query
    listado = paginator(query, request)

    #if no iface is found, it shows advanced iface info
    if len(listado.object_list) == 0:
        try:
            ipaddr.IPv4Address(query_sent)
            vlan = Iface.find_vlan(query_sent)
            eirs = Iface.excluded_in_ranges(ip=query_sent, vlan=vlan)
            conflicts = ConflictingIP.objects.filter(ip=query_sent)
            return render_to_response(
                "iface/adv_info.html",
                {
                    "vlan": vlan,
                    "eirs": eirs,
                    "ip": query_sent,
                    "conflicts": conflicts,
                },
                context_instance=RequestContext(request)
            )
        except Exception:
            pass

#    if len(listado.object_list) == 1:
#        obj = listado.object_list[0]
#        return redirect(reverse("iface", args=[obj.pk, ]))
    return render_to_response(
        'iface/list.html',
        {"listado": listado},
        context_instance=RequestContext(request))


@user_passes_test(lambda u: u.is_staff)
@csrf_exempt
def edit_by_machine(request, id):
    print "id iface: %s" % id
    if id is None:
        obj = Iface()

    else:
        obj = get_object_or_404(Iface, pk=id)
    wrong = False
    if request.method == "POST":
        form = IfaceByMachineForm(request.POST, instance=obj)
        try:
            if form.is_valid():
                machine = request.POST["machines"]
                obj = form.save(commit=False)
                obj.save()
                try:
                    obj.machines.add(machine)
                except IntegrityError:
                    pass

                messages.info(request, 'Iface successfully saved')
                return redirect(reverse('ifaces_by_machine', args=[machine, ]))

            wrong = True
            messages.error(request, "Wrong fields!")
        except Exception, ex:
            return HttpResponseServerError(ex)

    else:
        try:
            machine = request.GET["machines"]
            form = IfaceByMachineForm(instance=obj, initial={"machines": [machine]})
        except MultiValueDictKeyError:
            form = IfaceByMachineForm(instance=obj)

    response = render_to_response(
        'iface/edit_by_machine.html',
        {
            'form': form,
            "id": obj.pk if obj.pk is not None else "",
            "obj": obj,
        },
        context_instance=RequestContext(request))
    if wrong:
        response["FORM_VALIDATE"] = "false"
        for msg in messages.get_messages(request):
            response["FORM_MSGS"] = msg

    return response


@user_passes_test(lambda u: u.is_staff)
def edit_short_old(request, id):
    if id is None:
        obj = Iface()
    else:
        obj = get_object_or_404(Iface, pk=id)

    if request.method == "POST":
        form = IfaceShortForm(request.POST, instance=obj)

        if form.is_valid():
            obj = form.save(commit=False)
            obj.save()
            messages.info(request, 'Iface successfully saved')
            return redirect(reverse('iface', args=[obj.pk]))
        else:
            messages.error(request, "Wrong fields!")

    else:
        form = IfaceShortForm(request.GET, instance=obj)

    return render_to_response(
            'iface/edit_form.html',
            {
                'form': form,
                 "id": obj.pk,
                 "obj": obj,
             },
            context_instance=RequestContext(request))


@user_passes_test(lambda u: u.is_staff)
def edit(request, id=None):

    if id is None:
        obj = Iface()
    else:
        obj = get_object_or_404(Iface, pk=id)

    if request.method == "POST":
        form = IfaceForm(request.POST, instance=obj)

        if form.is_valid():

            obj = form.save(commit=False)
            obj.save()
            if "machines" in form.cleaned_data:
                obj.machines.clear()
                cd = form.cleaned_data["machines"]
                for machine in form.cleaned_data["machines"]:
                    obj.machines.add(machine)

            messages.info(request, 'Iface successfully saved')
            return redirect(reverse('iface', args=[obj.pk]))

        messages.error(request, "Wrong fields!")

    else:
        form = IfaceForm(initial=request.GET, instance=obj)

    return render_to_response(
            'iface/edit.html',
            {
            'form': form,
             "id": obj.pk,
             "obj": obj,
             },
            context_instance=RequestContext(request))



from naman.core.models import Machine, Iface, VLanConfig, ConflictingIP, Service
from django.forms import ModelForm


class VLanConfigForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(VLanConfigForm, self).__init__(*args, **kwargs)
        self.fields["machine"].widget.attrs = {"style": "display:none"}
        for name, f in self.fields.items():
           # f.widget.attrs = {'required': 'true'}

            f.widget.attrs['placeholder'] = "%s ..." % f.label
            if f.widget.__class__.__name__ != "CheckboxInput":
                f.widget.attrs["class"] = "form-control"

    class Meta:
        model = VLanConfig
        exclude = "vlans"


class ConflictingIPForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(ConflictingIPForm, self).__init__(*args, **kwargs)
        for name, f in self.fields.items():
           # f.widget.attrs = {'required': 'true'}

            f.widget.attrs['placeholder'] = "%s ..." % f.label
            if f.widget.__class__.__name__ != "CheckboxInput":
                f.widget.attrs["class"] = "form-control"
    class Meta:
        model = ConflictingIP



class MachineForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(MachineForm, self).__init__(*args, **kwargs)
        for name, f in self.fields.items():
           # f.widget.attrs = {'required': 'true'}
            f.widget.attrs['placeholder'] = "%s ..." % f.label
            if f.widget.__class__.__name__ != "CheckboxInput":
                f.widget.attrs["class"] = "form-control"

    def clean(self):
        cleaned_data = super(MachineForm, self).clean()
        hostname = cleaned_data.get("hostname")
        mtype = cleaned_data.get("mtype")
        role = cleaned_data.get("role")
        env = cleaned_data.get("environment")
        os = cleaned_data.get("operating_system")

        if hostname in (None, ""):
            if mtype is None or not mtype.auto_name:
                self._errors["mtype"] = self.error_class(
                    ["Hostname must be filled if mtype is not auto_named"])
                try:
                    del cleaned_data["mtype"]
                except KeyError:
                    pass

            if role is None:
                self._errors["role"] = self.error_class(
                    ["Role must be filled if hostname is empty"]
                )
                try:
                    del cleaned_data["role"]
                except KeyError:
                    pass

            if env is None:
                self._errors["environment"] = self.error_class(
                    ["Environment must be filled if hostname is empty"]
                )
                try:
                    del cleaned_data["environment"]
                except KeyError:
                    pass

            if os is None:
                self._errors["operating_system"] = self.error_class(
                    ["Operating system must be filled if hostname is empty"]
                )
                try:
                    del cleaned_data["operating_system"]
                except KeyError:
                    pass

        return cleaned_data

    class Meta:
        model = Machine


class IfaceByMachineForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(IfaceByMachineForm, self).__init__(*args, **kwargs)
        #self.fields["machines"].choices = Iface.objects.filter(id)
        #self.fields["machines"].widget.attrs = {"style": "display:none"}
        for name, f in self.fields.items():
           # f.widget.attrs = {'required': 'true'}
            f.widget.attrs['placeholder'] = "%s ..." % f.label
            if f.widget.__class__.__name__ != "CheckboxInput":
                f.widget.attrs["class"] = "form-control"

    class Meta:
        model = Iface
        fields = ("machines", "ip", "dhcp", "vlan", "mac", )

class ServiceForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(ServiceForm, self).__init__(*args, **kwargs)

        for name, f in self.fields.items():
            print name
           # f.widget.attrs = {'required': 'true'}
            f.widget.attrs['placeholder'] = "%s ..." % f.label
            if f.widget.__class__.__name__ != "CheckboxInput":
                f.widget.attrs["class"] = "form-control"

    class Meta:
        model = Service
        fields = ('name', 'iface', 'description')



class IfaceForm(ModelForm):
    def __init__(self, *args, **kwargs):
        super(IfaceForm, self).__init__(*args, **kwargs)
        self.fields["mask"].widget.attrs = {"disabled": ""}
        for name, f in self.fields.items():
           # f.widget.attrs = {'required': 'true'}
            f.widget.attrs['placeholder'] = "%s ..." % f.label
            if f.widget.__class__.__name__ != "CheckboxInput":
                f.widget.attrs["class"] = "form-control"

    class Meta:
        model = Iface


class IfaceShortForm(IfaceForm):
    class Meta:
        model = Iface
        exclude = ("nat", "gw", "mask", "name", "comments", "nat", "virtual", )



from naman.core.models import Machine
from forms import MachineForm
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from naman.core.tools.views import paginator
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.contrib import messages
from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponse, HttpResponseServerError
import logging


@user_passes_test(lambda u: u.is_staff)
def delete(request, id):
    try:
        get_object_or_404(Machine, pk=id).delete()
        return HttpResponse("OK")
    except Exception as er:
        logging.error("Exception in machine deletion: %s" % er)
        return HttpResponseServerError("An error has ocurred.")


@user_passes_test(lambda u: u.is_staff)
def listado(request):
    #TODO change from icontains to iregex
    if "query_machine" in request.GET and request.GET["query_machine"] != "":
        query = Machine.objects.filter(
            Q(hostname__icontains=request.GET["query_machine"]) |
            Q(dns_zone__name__icontains=request.GET["query_machine"])|
            Q(mtype__name__icontains=request.GET["query_machine"])|
            Q(environment__code__icontains=request.GET["query_machine"])
            )
        print query.query
    else:
        query = Machine.objects.all()

    listado = paginator(query, request)

    if len(listado.object_list) == 1:
        obj = listado.object_list[0]
        return redirect(reverse("machine", args=[obj.pk, ]))
    return render_to_response(
        'machine/list.html',
        {"listado": listado},
        context_instance=RequestContext(request))


@user_passes_test(lambda u: u.is_staff)
def edit(request, id=None):
#from django.forms.util import ErrorList
    if id is None:
        obj = Machine()
    else:
        obj = get_object_or_404(Machine, pk=id)

    wrong = False
    if request.method == "POST":
        form = MachineForm(request.POST, instance=obj)

        try:
            if form.is_valid():
                print "is valid!"
                obj = form.save(commit=False)
                obj.save()
                messages.info(request, 'Machine successfully saved')
                return redirect(reverse('machine', args=[obj.pk]))

            wrong = True
            messages.error(request, "Wrong fields!")

        except Exception as ex:
            print "exception: %s" % ex
            messages.error(request, ex)
    else:
        form = MachineForm(instance=obj)

    response = render_to_response(
            'machine/edit.html',
            {
            'form': form,
             "id": obj.pk,
             "obj": obj,
             },
            context_instance=RequestContext(request))
    if wrong:
        response.status_code = 400
    return response



import ldap
import os
import re
import datetime

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.models import Group
from django.contrib.auth.backends import ModelBackend #Need to include this as it has been disabled


#Need to have this class inherit from ModelBackend to have permissions inherit to groups.
class ActiveDirectoryAuthenticationBackend(ModelBackend):

    """
    This authentication backend authenticates against Active directory.
    It updates the user objects according to the settings in the AD and is
    able to map specific groups to give users admin rights.
    """

    def __init__(self):
        """ initialise a debuging if enabled """
        self.debug = settings.AD_DEBUG
        if len(settings.AD_DEBUG_FILE) > 0 and self.debug:
            self.debugFile = settings.AD_DEBUG_FILE
            # is the debug file accessible?
            if not os.path.exists(self.debugFile):
                open(self.debugFile,'w').close()
            elif not os.access(self.debugFile, os.W_OK):
                raise IOError("Debug File is not writable")
        else:
            self.debugFile = None

    def authenticate(self,username=None,password=None):
        try:
            if len(password) == 0:
                return None
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE,settings.AD_CERT_FILE)
            l = ldap.initialize(settings.AD_LDAP_URL)
            l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)
            binddn = "%s@%s" % (username,settings.AD_NT4_DOMAIN)
            l.simple_bind_s(binddn,password)
            l.unbind_s()
            return self.get_or_create_user(username,password)

        except ImportError:
            self.debug_write('import error in authenticate')
        except ldap.INVALID_CREDENTIALS:
            self.debug_write('%s: Invalid Credentials' % username)

    def get_or_create_user(self, username, password):
        """ create or update the User object """
        # get user info from AD
        userInfo = self.get_user_info(username, password)

        # is there already a user?
        try:
            user = User.objects.get(username=username)

        except User.DoesNotExist:
            if userInfo is not None:
                user = User(username=username, password=password)
                self.debug_write('create new user %s' % username)
                user.save() #Creates initial user so groups can be manipulated
            else:
                return None

        ## update the user objects
        # group mapping for the access rights

        user.groups.clear() #Clears user groups in case of changes

        for group in userInfo['idGroup']:  #Adds all groups user is member of

            g, created = Group.objects.get_or_create(name=group)

            user.groups.add(g)

        if userInfo['isAdmin']:

            user.is_staff = True
            user.is_superuser = True
        elif userInfo:

            user.is_staff = True
            user.is_superuser = False
        else:
            user.is_active = False

        # personal data
        user.first_name = userInfo['first_name']
        user.last_name = userInfo['last_name']
        user.email = userInfo['mail']

        # cache the AD password
        user.set_password('ldap authenticated') #puts in a dummy PW doesn't matter but use this to put a hash in

        user.save()

        return user

    def get_user(self, user_id):
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None

    def debug_write(self,message):
        """ handle debug messages """
        if self.debugFile is not None:
            fObj = open(self.debugFile, 'a')
            now = datetime.datetime.now().strftime('%Y-%m-%d %H:%M')
            fObj.write("%s\t%s\n" % (now,message))
            fObj.close()

    def check_group(self,membership):
        """ evaluate ADS group memberships """
        self.debug_write("Evaluating group membership")
        isValid = False
        isAdmin = False
        idGroup = []
        pattern = re.compile(r'^CN=(?P<groupName>[\w|\d|\s\(\)\-_]+),OU=')  #Our AD groups were mirrored with the Django groups but prefaced with "ID " (notice the space)

        for group in membership:

            groupMatch = pattern.match(group)
            if groupMatch:
                thisGroup = groupMatch.group('groupName')
                idGroup.append(thisGroup)
         #        print "mwetemos %s" % thisGroup
                if thisGroup in settings.AD_MEMBERSHIP_REQ:
                    isValid = True
                if thisGroup in settings.AD_MEMBERSHIP_ADMIN:
                    isAdmin = True

        if isAdmin:
            self.debug_write('is admin user')
        elif isValid:
            self.debug_write('is normal user')
        else:
            self.debug_write('does not have the AD group membership needed')

        return isAdmin, isValid, idGroup

    def get_user_info(self, username, password):
        """ get user info from ADS to a dictionary """
        try:
            userInfo = {
            'username' : username,
            'password' : password,
            }
            # prepare LDAP connection
            ldap.set_option(ldap.OPT_X_TLS_CACERTFILE,settings.AD_CERT_FILE)
            ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT,ldap.OPT_X_TLS_NEVER) #This required to avoid CN mismatches
            ldap.set_option(ldap.OPT_REFERRALS,0) # DO NOT TURN THIS OFF OR SEARCH WON'T WORK!

            # initialize
            self.debug_write('ldap.initialize...')
            l = ldap.initialize(settings.AD_LDAP_URL)
            l.set_option(ldap.OPT_PROTOCOL_VERSION, 3)

            # bind
            binddn = "%s@%s" % (username,settings.AD_NT4_DOMAIN)
            self.debug_write('ldap.bind %s' % binddn)
            l.bind_s(binddn,password)

            # search
            self.debug_write('search...')
            result = l.search_ext_s(settings.AD_SEARCH_DN,ldap.SCOPE_SUBTREE,"sAMAccountName=%s" % username,settings.AD_SEARCH_FIELDS)[0][1]
            self.debug_write("results in %s" % result)

            # Validate that they are a member of review board group
            if result.has_key('memberOf'):
                membership = result['memberOf']
            else:
                self.debug_write('AD user has no group memberships')
                return None

            # user ADS Groups
            isAdmin, isValid, idGroup = self.check_group(membership)

            if not isValid:
                return None

            userInfo['isAdmin'] = isAdmin
            userInfo['idGroup'] = idGroup
            self.debug_write(idGroup)

            # get user info from ADS
            # get email
            if result.has_key('mail'):
                mail = result['mail'][0]
            else:
                mail = ""

            userInfo['mail'] = mail
            self.debug_write("mail=%s" % mail)

            # get surname
            if result.has_key('sn'):
                last_name = result['sn'][0]
            else:
                last_name = None

            userInfo['last_name'] = last_name
            self.debug_write("last_name=%s" % last_name)

            # get display name
            if result.has_key('givenName'):
                first_name = result['givenName'][0]
            else:
                first_name = None

            userInfo['first_name'] = first_name
            self.debug_write("first_name=%s" % first_name)

            # LDAP unbind
            l.unbind_s()
            return userInfo

        except Exception, e:
            self.debug_write("exception caught!")
            self.debug_write(e)
            return None



from django.shortcuts import render_to_response, get_object_or_404
from naman.core.models import ConflictingIP
from django.http import HttpResponse, HttpResponseServerError
from django.template import RequestContext
from forms import ConflictingIPForm
from django.contrib.auth.decorators import user_passes_test


@user_passes_test(lambda u: u.is_staff)
def edit_modal(request, id=None):

    if id is None:
        obj = ConflictingIP()
    else:
        obj = get_object_or_404(ConflictingIP, pk=id)

    wrong = False
    if request.method == "POST":
        form = ConflictingIPForm(request.POST, instance=obj)

        try:
            if form.is_valid():
                print "is valid!"
                obj = form.save(commit=False)
                obj.save()
                return HttpResponse("OK")

            wrong = True

        except Exception, ex:
            print "exception: %s" % ex
            return HttpResponseServerError(ex)
    else:
        form = ConflictingIPForm(instance=obj)

    return render_to_response(
        "conflicting_ip/edit_modal.html",
        {
        'form': form,
         "id": obj.pk,
         "obj": obj,
         },
        context_instance=RequestContext(request))
    if wrong:
        response.status_code = 400
    return response



# -*- coding: utf-8 -*-
from naman.core.models import *
from django.db import connections, DatabaseError
redip = connections['redip'].cursor()
bahamas = connections['bahamas'].cursor()


class Command(BaseCommand):
    args = ''
    help = """

"""

    def handle(self, *args, **options):
        print args
        fo = open(args[0], "r")

   #     dns_zone = DNSZone.objects.get(name="undefined")
   #     environment = Environment.objects.get(code="unde")
   #     role = Role.objects.get(code="unde")
   #     os = OperatingSystem.objects.get(code="unde")
   #     mtype = MType.objects.get(name="imported")
        self.stdout.write("Aprovisionamos las vlanes")
        query = "select * from vlans"
        bahamas.execute(query)
        for row in bahamas.fetchall():
            VLan.objects.get_or_create(
                name=row[3],
                mask=row[4],
                ip=row[10],
                tag=0,
                gw=row[1],
            )

        self.stdout.write("Aprovisionamos las maquinas")
        for table in fo.readlines():
            table = table.replace("\n", "")
            self.stdout.write("Trabajamos con la tabla: %s" % table)
            query = "SELECT systemName, location FROM %s" % table
            self.stdout.write("query: %s" % query)

            try:
                redip.execute(query)
            except UnicodeDecodeError:
                self.stderr.write("ERROR: Tabla %s con fallos unicode" % table)
                continue

            for row in redip.fetchall():
                if row[0] not in ("", None):
                    try:
                        location = row[1]
                    except Exception:
                        location = None

                    if row[0] in (None, ""):
                        continue
                    print "metemos la maquina: %s" % row[0]
                    try:
                        m = Machine.objects.get(hostname=row[0])
                    except Machine.DoesNotExist:
                        Machine(hostname=row[0].lower()).save()


        fo.seek(0)
        self.stdout.write("Aprovisionamos las interfaces")
        for table in fo.readlines():
            hay_nat = True
            table = table.replace("\n", "")
            self.stdout.write("Trabajamos con la tabla: %s" % table)
            try:
                vlan = VLan.objects.get(name=table)
            except VLan.DoesNotExist:
                self.stderr.write("El fichero de tablas muestra la tabla %s, pero no existe" % table )
                continue

            query = "SELECT address, systemName, comments, nat FROM %s where status= 'Used'" % table

            try:
                redip.execute(query)

            except DatabaseError:
                query = "SELECT address, systemName, comments FROM %s where status= 'Used'" % table
                redip.execute(query)
                hay_nat = False
            except UnicodeDecodeError:
                self.stderr.write("ERROR: Tabla %s con fallos unicode" % table)
                continue

            for iface in redip.fetchall():
                machine = Machine.objects.get(hostname=iface[1])
                nat = row[3] if hay_nat else None
                niface = Iface(
                    name="migrated",
                    vlan=vlan,
                    ip=iface[0],
                    comments=iface[2],
                    nat=nat)
                niface.save()
                niface.machines.add(machine)

        fo.close()




from django_messages.models import Message
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required


@login_required
def avisa_admin(msg, user=None):
    try:
        admin = User.objects.get(username=settings.MANAGERS)
        Message(
            subject="AVISO del sistema",
            body=msg,
            recipient=admin,
            sender=user if user != None else admin,
            ).save()
        return True
    except Exception, ex:
        return False


def week_day(day_num):
    semana = ["Lunes", "Martes", "Miercoles", "Jueves", "Viernes", "Sabado", "Domingo"]
    return semana[day_num]



from naman.core.models import Service
from django.contrib.auth.decorators import user_passes_test
from django.shortcuts import get_object_or_404, redirect, render_to_response
from naman.web.forms import ServiceForm
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.template import RequestContext


@user_passes_test(lambda u: u.is_staff)
def edit(request, id=None):
#from django.forms.util import ErrorList
    if id is None:
        obj = Service()
    else:
        obj = get_object_or_404(Service, pk=id)

    wrong = False
    if request.method == "POST":
        form = ServiceForm(request.POST, instance=obj)

        try:
            if form.is_valid():
                print "is valid!"
                obj = form.save(commit=False)
                obj.save()
                messages.info(request, 'Service successfully saved')
                return redirect(reverse('service', args=[obj.pk]))

            wrong = True
            messages.error(request, "Wrong fields!")

        except Exception as ex:
            print "exception: %s" % ex
            messages.error(request, ex)
    else:
        form = ServiceForm(instance=obj)

    response = render_to_response(
            'service/edit.html',
            {
            'form': form,
             "id": obj.pk,
             "obj": obj,
             },
            context_instance=RequestContext(request))
    if wrong:
        response.status_code = 400
    return response


from models import (
    Project,
    Machine,
    Iface,
    DNSZone,
    Environment,
    VLan,
    Role,
    OperatingSystem,
    MType,
    ExcludedIPRange,
    Service,
    VLanConfig,
    )
from rest_framework.views import APIView
from django.views.decorators.csrf import csrf_exempt
from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from django.http import HttpResponseBadRequest, Http404
from rest_framework import status
import simplejson
from serializers import (
    ProjectSerializer,
    MachineSerializer,
    IfaceSerializer,
    DNSZoneSerializer,
    EnvironmentSerializer,
    VLanSerializer,
    RoleSerializer,
    OperatingSystemSerializer,
    MTypeSerializer,
    ExcludedIPRangeSerializer,
    ServiceSerializer,
    VLanConfigSerializer)
    
        
class VLanConfigList(APIView):
    """
    List all snippets, or create a new snippet.
    """
    def get(self, request, format=None):
        vlanconfigs = VLanConfig.objects.all()
        serializer = VLanConfigSerializer(vlanconfigs, many=True)
        return Response(serializer.data)
    @csrf_exempt
    def post(self, request, format=None):
        try:
            serializer = VLanConfigSerializer(data=request.DATA)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data, status=status.HTTP_201_CREATED)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except Exception, ex:
            
                return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
    
class VLanConfigDetail(APIView):
    """
    Retrieve, update or delete a snippet instance.
    """
    def get_object(self, pk):
        try:
            return VLanConfig.objects.get(pk=pk)
        except VLanConfig.DoesNotExist:
            raise Exception("Object not found")

    def get(self, request, pk, format=None):
        try:
            vlanconfig = self.get_object(pk)
            serializer = VLanConfigSerializer(vlanconfig)
            return Response(serializer.data)
        except Exception, ex:
            return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
    def put(self, request, pk, format=None):
        try:
            vlanconfig = self.get_object(pk)
            serializer = VLanConfigSerializer(vlanconfig, data=request.DATA)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except Exception, ex:
            return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
    def delete(self, request, pk, format=None):
        try:
            snippet = self.get_object(pk)
            snippet.delete()
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Exception, ex:
            return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
        
class MachineList(APIView):
    """
    List all snippets, or create a new snippet.
    """
    def get(self, request, format=None):
        machines = Machine.objects.all()
        serializer = MachineSerializer(machines, many=True)
        return Response(serializer.data)
    @csrf_exempt
    def post(self, request, format=None):
        try:
            serializer = MachineSerializer(data=request.DATA)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data, status=status.HTTP_201_CREATED)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except Exception, ex:
            
                return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
    
class MachineDetail(APIView):
    """
    Retrieve, update or delete a snippet instance.
    """
    def get_object(self, pk):
        try:
            return Machine.objects.get(pk=pk)
        except Machine.DoesNotExist:
            raise Http404(
                content=simplejson.dumps({"errors": ["Object not found", ]}),
                content_type="application/json",
                          )

    def get(self, request, pk, format=None):
        machine = self.get_object(pk)
        serializer = MachineSerializer(machine)
        return Response(serializer.data)

    def put(self, request, pk, format=None):
        machine = self.get_object(pk)
        serializer = MachineSerializer(machine, data=request.DATA)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, pk, format=None):
        machine = self.get_object(pk)
        machine.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)
    
    
class IfaceList(APIView):
    """
    List all snippets, or create a new snippet.
    """
    def get(self, request, format=None):
        machines = Iface.objects.all()[:3]
        serializer = IfaceSerializer(machines, many=True)
        return Response(serializer.data)
    @csrf_exempt
    def post(self, request, format=None):
        try:
            serializer = IfaceSerializer(data=request.DATA)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data, status=status.HTTP_201_CREATED)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except Exception, ex:
            
                return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
    
class IfaceDetail(APIView):
    """
    Retrieve, update or delete a snippet instance.
    """
    def get_object(self, pk):
        try:
            return Iface.objects.get(pk=pk)
        except Iface.DoesNotExist:
            raise Http404("Object not found")

    def get(self, request, pk, format=None):
        try:
            machine = self.get_object(pk)
            serializer = IfaceSerializer(machine)
            return Response(serializer.data)
        except Exception, ex:            
                return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
            

    def put(self, request, pk, format=None):
        try:
            machine = self.get_object(pk)
            serializer = IfaceSerializer(machine, data=request.DATA)
            if serializer.is_valid():
                serializer.save()
                return Response(serializer.data)
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
        except Exception, ex:            
                return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )

    def delete(self, request, pk, format=None):
        try:
            machine = self.get_object(pk)
            machine.delete()
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Exception, ex:            
                return HttpResponseBadRequest(
                    content=simplejson.dumps({"errors": [str(ex), ]}),
                    content_type="application/json",
                )
    
class ServiceViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """
    queryset = Service.objects.all()
    serializer_class = ServiceSerializer


class ExcludedIPRangeViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """
    queryset = ExcludedIPRange.objects.all()
    serializer_class = ExcludedIPRangeSerializer


class OperatingSystemViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """
    queryset = OperatingSystem.objects.all()
    serializer_class = OperatingSystemSerializer


class ProjectViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed or edited.
    """
    queryset = Project.objects.all()
    serializer_class = ProjectSerializer


class MTypeViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = MType.objects.all()
    serializer_class = MTypeSerializer


class DNSZoneViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = DNSZone.objects.all()
    serializer_class = DNSZoneSerializer


class VLanViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = VLan.objects.all()
    serializer_class = VLanSerializer


class EnvironmentViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows groups to be viewed or edited.
    """
    queryset = Environment.objects.all()
    serializer_class = EnvironmentSerializer


class RoleViewSet(viewsets.ModelViewSet):
    queryset = Role.objects.all()
    serializer_class = RoleSerializer




import logging
import re

from django.db import models
from django.db import transaction
from django.core.exceptions import ObjectDoesNotExist
import ipaddr
import simplejson
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.db.models.signals import post_save





# Get an instance of a logger
from naman.core.pypelib import RuleTable

logger = logging.getLogger(__name__)


class MoreThanOneIfacePerVlanError(Exception):
    pass


class IfaceSequence(models.Model):
    machine = models.ForeignKey("Machine")
    vlan = models.ForeignKey("VLan")
    last_number = models.IntegerField(default=0)

    @property
    def incr(self, ):
        self.last_number += 1
        self.save()
        return self.last_number


class HostnameSequence(models.Model):
    prefix = models.CharField(max_length=11)
    last_number = models.IntegerField(default=0)

    @property
    def next(self, ):
        return self.last_number + 1

    @property
    def incr(self, ):
        self.last_number += 1
        self.save()
        return self.last_number

    def __unicode__(self, ):
        return u"%s(%s)" % (self.prefix, self.last_number)


class CICaracteristic(models.Model):
    code = models.CharField(max_length=4)
    description = models.TextField()

    def __unicode__(self, ):
        return u"%s" % self.code

    class Meta:
        abstract = True


class DNSZone(models.Model):
    name = models.CharField(max_length=100)

    def __unicode__(self, ):
        return u"%s" % self.name

    def save(self, *args, **kwargs):
        new = False
        if self.pk is None:
            new = True
        super(DNSZone, self).save(*args, **kwargs)
        if new:
            logger.info("New DNSZone created: %s" % self)


class MType(models.Model):
    name = models.CharField(max_length=20)
    auto_name = models.BooleanField(default=True)
    has_serial = models.BooleanField(default=True)

    def __unicode__(self, ):
        return u"%s" % self.name


class Environment(models.Model):
    code = models.CharField(max_length=4, blank=True)
    description = models.TextField()
    backup_vlans = models.ManyToManyField('VLan', null=True, blank=True)
    service_vlans = models.ManyToManyField(
        'VLan',
        related_name='environments',
        null=True,
        blank=True)

    def __unicode__(self, ):
        return u"%s" % self.description
    
    def to_pypelib(self):
        return str(self.code)


class Role(CICaracteristic):
    needs_backup_vlan = models.BooleanField(default=False)
    
    def to_pypelib(self):
        return u"%s" % self.code


class OperatingSystem(CICaracteristic):
    def __unicode__(self, ):
        return u"%s" % self.description
    
    


class ConflictingIP(models.Model):
    ip = models.GenericIPAddressField(unique=True)
    comments = models.TextField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)

    def __unicode__(self, ):
        return u"%s" % self.ip


class ExcludedIPRange(models.Model):
    first = models.IPAddressField()
    last = models.IPAddressField()
    vlan = models.ForeignKey("VLan", related_name="excluded_ranges")

    class ExcludedIPError(Exception):
        pass

    def __unicode__(self, ):
        return "vlan: %s (%s - %s)" % (self.vlan, self.first, self.last)

    @property
    def as_tuple(self, ):
        return (self.first, self.last)

    def in_range(self, ip):
        if not isinstance(ip, ipaddr.IPv4Address):
            ip = ipaddr.IPv4Address(ip)

        if not isinstance(self.first, ipaddr.IPv4Address):
            self.first = ipaddr.IPv4Address(self.first)

        if not isinstance(self.last, ipaddr.IPv4Address):
            self.last = ipaddr.IPv4Address(self.last)

        if ip in self.as_tuple:
            return True

        if ip > self.first and ip < self.last:
            return True
        return False


@receiver(pre_save, sender=ExcludedIPRange)
def pre_save_excludediprange(sender, instance, **kwargs):

        if not instance.vlan.is_ip_valid(instance.first):
            raise ipaddr.AddressValueError("First IP is not correct for vlan")

        if not instance.vlan.is_ip_valid(instance.last):
            raise ipaddr.AddressValueError("Last IP is not correct for vlan")


class VLanManager(models.Manager):

    def get_from_ip(self, ip):

        for vlan in self.all():
            if vlan.is_ip_valid(ip):
                return vlan
        return None


class VLan(models.Model):
    name = models.CharField(max_length=10)
    tag = models.IntegerField()
    ip = models.IPAddressField()
    gw = models.IPAddressField()
    mask = models.IntegerField()
    management_purpose = models.BooleanField(default=False)
    provisioning_purpose = models.BooleanField(default=False)
    dhcp = models.BooleanField(default=True)
    #general_purpose_service = models.BooleanField(default=False)

    class Meta:
        ordering = ("name", )

    class NoFreeIPError(Exception):
        pass

    def __unicode__(self, ):
        return u"%s(%s)" % (self.name, self.tag)

    @property
    def info(self, ):
        hosts = [x for x in self.network.iterhosts()]
        return u"%s; first_ip: %s; last_ip: %s; num_hosts: %s" % (
            self,
            hosts[0],
            hosts[-1],
            len(hosts),
        )

    def is_ip_valid(self, ip):
        """ Calculates if an IP belongs to the vlan addressing scope"""

        if not isinstance(ip, ipaddr.IPv4Address):
            ip = ipaddr.IPv4Address(ip)
        for nip in self.network.iterhosts():
            if nip == ip:
                return True
        return False

    @property
    def network(self, ):
        try:
            return ipaddr.IPv4Network("%s/%s" % (self.ip, self.mask))
        except Exception:
            raise ValueError(
                "VLan.network: Can't calculate vlan network due to vlan %s misconfiguration" % self.name)


    @property
    def has_free_ip(self, ):
        return True if self.get_ip() else False

    def get_ip(self, ):
        """ searches and returns a free IP, respects excluded ranges and conflicting ips"""
        eranges = self.excluded_ranges
        #print "looking for an IP in scope: %s" % self.network

        for ip in self.network.iterhosts():
            #print "tryuing with: %s" % ip
            try:
                for erange in eranges.all():
                    if erange.in_range(ip):
                        raise ExcludedIPRange.ExcludedIPError
                #print "query: %s" % ConflictingIP.objects.filter(ip=ip).query
                if ConflictingIP.objects.filter(ip=str(ip)).exists():
                    continue
            except ExcludedIPRange.ExcludedIPError:
                continue
            try:
                Iface.objects.get(ip=str(ip))
            except ObjectDoesNotExist:
                return str(ip)

        return None

    def save(self, *args, **kwargs):
        new = False
        if self.pk is None:
            new = True
        super(VLan, self).save(*args, **kwargs)
        if new:
            logger.info("New VLAN created: %s" % self)


# Create your models here.
class Project(models.Model):
    name = models.CharField(max_length=200)
    code = models.CharField(max_length=20)
    dmz = models.ForeignKey('VLan', null=True, blank=True)
    service_vlans = models.ManyToManyField('VLan', related_name='projects')
    #machines = models.ManyToManyField("Machine")

    def __unicode__(self, ):
        return u"%s(%s)" % (self.name, self.code)

    def save(self, *args, **kwargs):
        new = False
        if self.pk is None:
            new = True
        super(Project, self).save(*args, **kwargs)
        if new:
            logger.info("New Project created: %s" % self)
            
    def to_pypelib(self):
        return str(self.code)


class Machine(models.Model):
    hostname = models.CharField(max_length=25, blank=True, null=True)
    dns_zone = models.ForeignKey(DNSZone, blank=True, null=True)
    environment = models.ForeignKey(Environment, blank=True, null=True)
    role = models.ForeignKey(Role, blank=True, null=True)
    operating_system = models.ForeignKey(
        OperatingSystem,
        blank=True,
        null=True)
    virtual = models.BooleanField(default=False)
    project = models.ForeignKey(
        Project,
        related_name="machines",
        null=True,
        blank=True)
    mtype = models.ForeignKey(MType, null=True, blank=True)
    location = models.CharField(max_length=50, null=True, blank=True)
    dmz_located = models.BooleanField(default=False)
    #close_to = models.ForeignKey(Machine)

    class Meta:
        ordering = ("hostname",)

    def initialize_hostname(self, ):
        if self.role is None:
            raise AttributeError("If no hostname is specified, role must be initialized")
        if self.operating_system is None:
            raise AttributeError("If no hostname is specified, operating system must be initialized")
        if self.environment is None:
            raise AttributeError("If no hostname is specified, environment must be initialized")
        hn = u"%s%s%s%s" % (
            self.role.code,
            self.project.code if self.project is not None else "",
            self.operating_system.code,
            self.environment.code if self.environment.code != "PRO" else "",
            )
        hn = hn.lower()
        if self.mtype.has_serial:
            hn = "%s%s" % (
                hn,
                HostnameSequence.objects.get_or_create(prefix=hn)[0].incr)
        self.hostname = hn

    def initialize_dnszone(self):

        try:
            if self.operating_system.code == "w":
                self.dns_zone = DNSZone.objects.get(name=".iberia.ib")
            else:
                self.dns_zone = DNSZone.objects.get(name=".ib")

            if self.environment.code == "lab":
                self.dns_zone = DNSZone.objects.get(name=".lab")
        except AttributeError:
            pass

    def save(self, *args, **kwargs):
        new = False
        print "save hostbname: %s" % self.hostname
        with transaction.commit_on_success():
            if self.pk is None:
                new = True
                if(self.mtype is not None and
                    self.mtype.auto_name and
                    self.hostname in (None, "")
                    ):
                    self.initialize_hostname()

                elif self.hostname is None or self.hostname == "":
                    raise AttributeError(
                        "Hostname missing for a non automatic one")
                self.initialize_dnszone()

            super(Machine, self).save(*args, **kwargs)
            if new:
                logger.info("New machine created: %s" % self)

    @property
    def fqdn(self, ):
        return u"%s%s" % (self.hostname, self.dns_zone)

    def __unicode__(self, ):
        return u"%s" % self.fqdn if not self.dns_zone is None else self.hostname

    def has_iface_on_vlan(self, vlan):
        return self.interfaces.filter(vlan=vlan).exists()
    
    def get_vlanconfig(self):
        if not self.vlan_configs.count():
            self.vlan_configs.append(VLanConfig())
        return self.vlan_configs.get() 


class Service(models.Model):
    name = models.CharField(max_length=255)
    description = models.TextField(null=True, blank=True)
    iface = models.ForeignKey("Iface", related_name="services") #Esto permite referenciar los servicios de un iface desde un objeto Iface

    def __unicode__(self, ):
        return u"%s" % self.name


class IfaceManager(models.Manager):

    def query_cmd(self, query_string):

        m = re.match("^(.+)=(.+)$", query_string)
        if not m:
            #print "no hay coincidencia"
            return None
        try:
            print "Se ha encontrado: %s" % m.group(2)
            queriable = {
                "name": self.filter(name__iregex=m.group(2)),
                "vlan": self.filter(vlan__name__iregex=m.group(2)),
                "machines": self.filter(machines__hostname__iregex=m.group(2)),
                "ip": self.filter(ip__iregex=m.group(2)),
                "gw": self.filter(gw__iregex=m.group(2)),
                "mask": self.filter(mask__iregex=m.group(2)),
                "comments": self.filter(comments__iregex=m.group(2)),
                "mac": self.filter(mac__iregex=m.group(2)),
                "nat": self.filter(nat__iregex=m.group(2)), }
            return queriable[m.group(1)]
        except KeyError:
            pass
        return None


class Iface(models.Model):
    objects = IfaceManager()
    PREFIX = "eth"

    name = models.CharField(max_length=11, blank=True)
    vlan = models.ForeignKey(VLan)
    ip = models.GenericIPAddressField(blank=True, unique=True, null=True)
    gw = models.GenericIPAddressField(blank=True, default="0.0.0.0")
    mask = models.IntegerField(blank=True)
    machines = models.ManyToManyField(
        Machine,
        related_name="interfaces",
        null=True,
        blank=True)
    comments = models.TextField(null=True, blank=True)
    mac = models.CharField(max_length=17, null=True, blank=True)
    nat = models.GenericIPAddressField(null=True, blank=True)
    virtual = models.BooleanField(default=False)
    dhcp = models.BooleanField(default=False)

    def to_json(self, ):
        return simplejson.dumps({
            "id": self.pk,
            "ip": self.ip,
            "vlan": self.vlan.pk,
            "dhcp": self.dhcp,
            })

    def __unicode__(self, ):
        return u"%s" % self.ip if not self.ip is None else self.name if self.name != ""  else "IFACE"



    @staticmethod
    def excluded_in_ranges(ip, vlan=None):

        if vlan is None:
            vlan = Iface.find_vlan(ip)
            if vlan is None:
                return []
        exclusions = []
        for eir in ExcludedIPRange.objects.filter(vlan=vlan):
            if eir.in_range(ip):
                exclusions.append(eir)
        return exclusions

    @staticmethod
    def find_vlan(ip):
        """
        Looksup a valid vlan for the passed ip.
        if no vlan found, return None
        """
        for vlan in VLan.objects.all():
            try:
                if vlan.is_ip_valid(ip):
                    return vlan
            except ValueError:
                logger.error("VLan %s might be misconfigured, thus can't check if ip %s belongs to it" % (vlan, ip))
                continue

        return None

    def save(self, *args, **kwargs):
        new = False
        with transaction.commit_on_success():
            if self.pk is None:
                new = True
                
                self.dhcp = self.vlan.dhcp               
                
                self.ip = self.vlan.get_ip() if self.ip in (None, "") and not self.dhcp else self.ip
                #If IP comes from user, check its valid for vlan
                if not self.dhcp and not self.vlan.is_ip_valid(self.ip): raise AttributeError("Ip %s is not valid for vlan %s" % (self.ip, self.vlan))
                                    
                self.gw = self.vlan.gw
                

            self.mask = self.vlan.mask

            super(Iface, self).save(*args, **kwargs)

            if new:
                logger.info("New Iface created: %s" % self)


COND_CHOICES = (
    ('environment', 'Machine environment'),
    ('project', 'Machine project'),
    ('role_needs_backup', 'Machine role needs backup'),
    ('dmz_located', 'Machine located in DMZ'),
    ("needs_backup", "Machine needs backup"),
    ('needs_management', "Machine needs management"),
)


class Conditional(models.Model):
    statement1 = models.CharField(max_length=60,choices=COND_CHOICES, default="environment")
    statement2 = models.CharField(max_length=60, null=True, blank=True)
    #ocurred = models.BooleanField(default=False)
    
    def __unicode__(self):
        return u"%s is %s" % (self.statement1, self.statement2)
    
    def to_pypelib(self):
        return u"(%s = %s)" % (str(self.statement1), str(self.statement2))


ACTION_CHOICES = (
    ("assign_backup_vlan", "Assign backup vlan"),
    ("assign_management_vlan", "Assign management vlan"),
    ("assign_provisioning_vlan", "Assign provisioning vlan"),
    ("assign_dmz_based_on_project", "Assign DMZ based on project"),
    ("assign_service_vlan_based_on_project", "Assign service vlan based on project"),
    ("assign_general_purpose_service_vlan", "Assign general purpose vlan"),
)

class Rule(models.Model):

    table = models.CharField(max_length=30)
    conditionals = models.ManyToManyField(Conditional, null=True, blank=True)
    #conditionals = models.CharField(max_length=65, choices=COND_CHOICES)
    action = models.CharField(max_length=65, choices=ACTION_CHOICES, null=True, blank=True)
    active = models.BooleanField(default=True)    
    terminal = models.BooleanField(default=False)
    
    def __unicode__(self, ):
        return "[%s] conditionals: %s; action: %s" % (
            self.active,

            self.conditionals.all(),
            self.action,
        )

    def to_pypelib(self, ):
        
        conds = self.conditionals.all()
        
        out = "if "
                
        if conds.count():
            out = "%s (" % out
            for c in conds:
                out = "%s %s && " % (out, c.to_pypelib())
                
            out = re.sub("&&\s$", ")", out)
        #print out
        #out = re.sub("&&$", "", out)
        if conds.count() == 0: out = "%s 1 = 1 " % out
        
        return "%s then accept %s do %s" % (out, "" if self.terminal else "nonterminal", self.action)
    
class VLanConfig(models.Model):

    machine = models.ForeignKey(Machine, related_name="vlan_configs")
    vlans = models.ManyToManyField(VLan, blank=True, null=True)
    needs_backup = models.BooleanField(default=True)
    needs_management = models.BooleanField(default=False)
    class Meta:
        verbose_name = "Configuracion de vlanes para maquina"
        verbose_name_plural = "Configuraciones de vlanes para maquinas"

    def __unicode__(self, ):
        out = u"VLan config for %s: [" % self.machine

        for vlan in self.vlans.all().order_by('name'):
            out = "%s %s," % (out, vlan)

        return u"%s]" % out

    def append_vlan(self, vlan):
        """ Adds a new vlan checking if it has free IPs """
        if not vlan.has_free_ip:
            raise VLan.NoFreeIPError(vlan)
        self.vlans.add(vlan)
        print("Vlan %s added" % vlan)


    def save(self, *args, **kwargs):

        new = (self.pk is None)

        super(VLanConfig, self).save(*args, **kwargs)

        
        
        
            
@receiver(post_save, sender=VLanConfig)
def post_save_vlanconfig(sender, instance, **kwargs):
    from naman.core.mappings import get_mappings
    from naman.core.pypelib.RuleTable import RuleTable


    if instance.machine.role is None:
        raise AttributeError("Machine %s has no role assigned" % instance.machine)
    if instance.machine.environment is None:
        raise AttributeError("Machine %s has no environment assigned" % instance.machine)

    table = RuleTable(
        "Backup, service and management",
        get_mappings(),
        "RegexParser",
        #rawfile,
        "RAWFile",
        None)

    table.setPolicy(False)
    for rule in Rule.objects.filter(active=True).order_by("pk"):
        table.addRule(rule.to_pypelib())
        
    logging.debug(table.dump())
    
    try:
        table.evaluate(instance.machine)
        logging.debug("Ha evaluado a True")
    except Exception, ex:
        import traceback
        print "Ha evaluado a False: %s" % traceback.format_exc()
        logging.debug("Table evaluated False")
        pass
    logging.debug("Vlan config saved, vlans: %s" % instance.vlans.all())
    for vlan in instance.vlans.all():
        iface = Iface(vlan=vlan)
        iface.save()
        iface.machines.add(instance.machine)


@receiver(pre_save, sender=VLanConfig)
def pre_save_vlanconfig(sender, instance, **kwargs):
    if instance.pk is None:
        #deleting possible previous vlanconfigs, only if its new
        VLanConfig.objects.filter(machine=instance.machine).delete()



from django.conf import settings
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.contrib import messages


def content_procesor_referer(request):

    return {
        "referer": request.META["HTTP_REFERER"] if "HTTP_REFERER" in request.META else "",
    }


def paginator(queryset, request, objs_per_page=None):
    paginator = Paginator(queryset, settings.RESULTS_PER_PAGE if objs_per_page == None else objs_per_page)
    page = request.GET['page'] if 'page' in request.GET and request.GET['page'] != None else 1

    try:
        return paginator.page(page)
    except PageNotAnInteger:
        messages.add_message(request, messages.ERROR, 'Invalid page, showing page number 1')
        return paginator.page(1)
    except EmptyPage:
        messages.add_message(request, messages.ERROR, 'Invalid page, showing last one')
        return paginator.page(paginator.num_pages)



import logging

from naman.core.pypelib import RuleTable

from naman.core.mappings import get_mappings
from naman.core.models import Rule
from django.conf import settings


def evaluate(machine):

    mappings = get_mappings()
    table = RuleTable(
        "Decide tunned temp",
        mappings,
        "RegexParser",
        #rawfile,
        "RAWFile",
        None)

    table.setPolicy(False)

    table.addRule("if heater_manual = 1  then accept do tune_to_confort")

    for rule in Rule.objects.filter(active=True, thermostat=True).order_by("pk"):
        table.addRule(rule.to_pypelib())

    if settings.DEBUG:
        table.dump()

    metaObj = {}

    try:
        table.evaluate(metaObj)
    except Exception:
        mappings["tune_to_economic"]()

    table1 = RuleTable(
        "Decide flame status",
        mappings,
        "RegexParser",
        #rawfile,
        "RAWFile",
        None)
    table1.addRule("if heater_on = 0 then deny")
    table1.addRule("if current_internal_temperature < tuned_temperature then accept")
    if settings.DEBUG:
        table1.dump()
        
    try:
        table1.evaluate(metaObj)
        try:
            mappings["start_flame"]()
        except Exception, ex:
            logging.error("ERROR: Cant start flame: %s" % ex)
    except Exception as e:
        try:
            mappings["stop_flame"]()
        except Exception, ex:
            logging.error("ERROR: Cant stop flame: %s" % ex)

